package Minimax;

/* Java KnockAbout Client
 * By : Maxime CHAMBREUIL
 * Student Id #:260067572
 */

class Distance {
   
    private Board myBoard;

    Distance(Board theBoard) {
	this.myBoard = theBoard;
    }
    
    private int getDistanceBetweenPosition(int x1, int y1, int x2, int y2) {
	int X,Y;
	
	if (x1 < x2) 
	    X = x2-x1;
	else 
	    X = x1-x2;
	if (y1 < y2) 
	    Y = y2-y1;
	else 
	    Y = y1-y2;
	
	int distance = X+Y;
	
	if (X == Y && (x1-x2)*(y1-y2) > 0) distance = distance/2; // same diagonal
	
	return distance;
    }
    
    // Return the minimal number of hex between an hex and the gutter
    private int getDistanceFromGutter(int x, int y){
	int currentDistance = 0;
	boolean hasMetTheGutter = false;
	
	if (myBoard.inGutter(x,y)) {
	    return currentDistance;
	}
	else {
	    while (!hasMetTheGutter){
		currentDistance++;
		if (myBoard.inGutter(x,y-currentDistance) || myBoard.inGutter(x-currentDistance,y-currentDistance) || myBoard.inGutter(x-currentDistance,y+currentDistance) 
		    || myBoard.inGutter(x,y+currentDistance) || myBoard.inGutter(x+currentDistance,y+currentDistance) || myBoard.inGutter(x+currentDistance,y-currentDistance) ) {
		    
		    hasMetTheGutter = true;
		}
	    }
	    return currentDistance;
	}
    }
    
    public int[] getFurthestDieFromGutter(int playerColor){
	Hex hex;   
	Die myDie;
	int currentDistance;
	int maxDistance = 0;
	int[] coord = new int[2];
	
	for (int i = 1;i < 12;i++) {
	    for (int j = 1;j < 12;j++) {
		if (myBoard.isValidCoord(i,j)) {
		    hex = myBoard.getHex(i,j);
		    
		    // The place is occupied by a right die
		    if (hex.isOccupied() && hex.getDie().getColor() == playerColor) {
			
			// I retrieve the distance between the die and the gutter
			currentDistance = getDistanceFromGutter(i,j);

			// I take the last die with the maximum distance
			if ( currentDistance > maxDistance) {
			    maxDistance = currentDistance;
			    coord[0] = i;
			    coord[1] = j;
			}
		    }
		}
	    }
	}
	return coord;
    }

    public int[] getMyNearestDieToADie(int myColor,int[] oppCoord){
	Die myDie;
	Hex hex;
	int currentDistance;
	int minDistance = 14;
	int[] coord = new int[2];
	
	for (int i = 0;i < 13;i++) {
	    for (int j = 0;j < 13;j++) {
		if (myBoard.isValidCoord(i,j)) {
		    hex = myBoard.getHex(i,j);
		    
		    // The place is occupied by an opponent die
		    if (hex.isOccupied() && hex.getDie().getColor() == myColor) {
				
			// I retrieve the distance between the die and the gutter
			currentDistance = getDistanceBetweenPosition(oppCoord[0],oppCoord[1],i,j);
			
			// I take the last die with the maximum distance
			if (currentDistance < minDistance){
			    minDistance = currentDistance;
			    coord[0] = i;
			    coord[1] = j;
			}
		    }
		}
	    }
	}
	return coord;
    }
}
